@@ -13,6 +13,8 @@ use safecast::*;
 use tc_btree::BTreeInstance;
 use tc_error::*;
 use tc_table::TableInstance;
+#[cfg(feature = "tensor")]
+use tc_tensor::TensorAccess;
 use tc_transact::fs::*;
 use tc_transact::lock::{Mutable, TxnLock};
 use tc_transact::{IntoView, Transact, Transaction, TxnId};
@@ -62,12 +64,13 @@ impl History {
 
     pub async fn append_put(
         &self,
-        txn_id: TxnId,
+        txn: Txn,
         path: TCPathBuf,
         key: Value,
         value: State,
     ) -> TCResult<()> {
-        let value = self.save_state(txn_id, value).await?;
+        let txn_id = *txn.id();
+        let value = self.save_state(txn, value).await?;
 
         debug!("History::append_put {} {} {} {}", txn_id, path, key, value);
 
@@ -76,7 +79,7 @@ impl History {
         Ok(())
     }
 
-    async fn save_state(&self, txn_id: TxnId, state: State) -> TCResult<Scalar> {
+    async fn save_state(&self, txn: Txn, state: State) -> TCResult<Scalar> {
         if state.is_ref() {
             return Err(TCError::bad_request(
                 "cannot update Chain with reference: {}",
@@ -84,10 +87,11 @@ impl History {
             ));
         }
 
+        let txn_id = *txn.id();
         match state {
             State::Collection(collection) => match collection {
                 Collection::BTree(btree) => {
-                    let hash: Id = btree.hash_hex(txn_id).await?.parse()?;
+                    let hash: Id = btree.hash_hex(&txn).await?.parse()?;
                     let schema = btree.schema().to_vec();
                     let classpath = BTreeType::default().path();
 
@@ -97,7 +101,7 @@ impl History {
                             .create_file(txn_id, hash.clone(), btree.class())
                             .await?;
 
-                        BTreeFile::copy_from(btree, file, txn_id).await?;
+                        BTreeFile::copy_from(btree, file, txn).await?;
                     }
 
                     Ok(OpRef::Get((
@@ -107,13 +111,13 @@ impl History {
                     .into())
                 }
                 Collection::Table(table) => {
-                    let hash: Id = table.hash_hex(txn_id).await?.parse()?;
+                    let hash: Id = table.hash_hex(&txn).await?.parse()?;
                     let schema = table.schema().clone();
                     let classpath = TableType::default().path();
 
                     if !self.dir.contains(&txn_id, &hash).await? {
                         let dir = self.dir.create_dir(txn_id, hash.clone()).await?;
-                        TableIndex::copy_from(table, dir, txn_id).await?;
+                        TableIndex::copy_from(table, dir, txn).await?;
                     }
 
                     Ok(OpRef::Get((
@@ -122,6 +126,35 @@ impl History {
                     ))
                     .into())
                 }
+
+                #[cfg(feature = "tensor")]
+                Collection::Tensor(Tensor::Dense(tensor)) => {
+                    let hash = tensor.hash_hex(&txn).await?.parse()?;
+                    let shape: Tuple<Value> = tensor
+                        .shape()
+                        .to_vec()
+                        .into_iter()
+                        .map(Value::from)
+                        .collect();
+
+                    let dtype = tc_value::ValueType::from(tensor.dtype()).path();
+                    let classpath = TensorType::Dense.path();
+
+                    if !self.dir.contains(&txn_id, &hash).await? {
+                        let file = self
+                            .dir
+                            .create_file(txn_id, hash.clone(), TensorType::Dense)
+                            .await?;
+
+                        DenseTensor::copy_from(tensor, file, txn).await?;
+                    }
+
+                    Ok(OpRef::Get((
+                        (hash.into(), classpath).into(),
+                        Value::Tuple(vec![shape.into(), dtype.into()].into()).into(),
+                    ))
+                    .into())
+                }
             },
             State::Scalar(value) => Ok(value),
             other if Scalar::can_cast_from(&other) => Ok(other.opt_cast_into().unwrap()),
@@ -264,7 +297,7 @@ impl History {
                         Mutation::Put(path, key, value) => {
                             other
                                 .resolve(txn, value)
-                                .and_then(|state| self.save_state(txn_id, state))
+                                .and_then(|state| self.save_state(txn.clone(), state))
                                 .and_then(|value| {
                                     if append {
                                         dest.append_put(
@@ -381,6 +414,12 @@ impl History {
                 let table = TableIndex::load(txn, schema, dir).await?;
                 Ok(Collection::Table(table.into()))
             }
+
+            #[cfg(feature = "tensor")]
+            CollectionType::Tensor(tt) => Err(TCError::not_implemented(format!(
+                "History::resolve instance of {}",
+                tt
+            ))),
         }
     }
 }
@@ -388,9 +427,10 @@ impl History {
 const SCHEMA: () = ();
 
 #[async_trait]
-impl Persist<fs::Dir, Txn> for History {
+impl Persist<fs::Dir> for History {
     type Schema = ();
     type Store = fs::Dir;
+    type Txn = Txn;
 
     fn schema(&self) -> &() {
         &SCHEMA
@@ -492,7 +532,7 @@ impl de::Visitor for HistoryVisitor {
         let mut i = 0u64;
         let txn = subcontext(i).await?;
 
-        if let Some(state) = seq.next_element::<State>(txn).await? {
+        if let Some(state) = seq.next_element::<State>(txn.clone()).await? {
             let (hash, block_data): (Bytes, Map<Tuple<State>>) = state
                 .try_cast_into(|s| TCError::bad_request("invalid Chain block", s))
                 .map_err(de::Error::custom)?;
@@ -506,7 +546,7 @@ impl de::Visitor for HistoryVisitor {
                 ));
             }
 
-            let mutations = parse_block_state(&history, txn_id, block_data)
+            let mutations = parse_block_state(&history, txn.clone(), block_data)
                 .map_err(de::Error::custom)
                 .await?;
 
@@ -540,7 +580,7 @@ impl de::Visitor for HistoryVisitor {
                 ));
             }
 
-            let mutations = parse_block_state(&history, txn_id, block_data)
+            let mutations = parse_block_state(&history, txn.clone(), block_data)
                 .map_err(de::Error::custom)
                 .await?;
 
@@ -555,7 +595,7 @@ impl de::Visitor for HistoryVisitor {
 
 async fn parse_block_state(
     history: &History,
-    txn_id: TxnId,
+    txn: Txn,
     block_data: Map<Tuple<State>>,
 ) -> TCResult<BTreeMap<TxnId, Vec<Mutation>>> {
     let mut mutations = BTreeMap::new();
@@ -571,7 +611,7 @@ async fn parse_block_state(
                 parsed.push(Mutation::Delete(path, key));
             } else if op.matches::<(TCPathBuf, Value, State)>() {
                 let (path, key, value) = op.opt_cast_into().unwrap();
-                let value = history.save_state(txn_id, value).await?;
+                let value = history.save_state(txn.clone(), value).await?;
                 parsed.push(Mutation::Put(path, key, value));
             } else {
                 return Err(TCError::bad_request(
