@@ -11,6 +11,8 @@ use log::debug;
 use tc_btree::BTreeView;
 use tc_error::*;
 use tc_table::TableView;
+#[cfg(feature = "tensor")]
+use tc_tensor::TensorView;
 use tc_transact::fs::Dir;
 use tc_transact::{IntoView, Transaction};
 use tcgeneric::{
@@ -23,18 +25,31 @@ use crate::txn::Txn;
 pub use tc_btree::BTreeType;
 pub use tc_table::TableType;
 
+#[cfg(feature = "tensor")]
+pub use tc_tensor::{DenseAccess, TensorType};
+
 pub type BTree = tc_btree::BTree<fs::File<tc_btree::Node>, fs::Dir, Txn>;
 pub type BTreeFile = tc_btree::BTreeFile<fs::File<tc_btree::Node>, fs::Dir, Txn>;
+
 pub type Table = tc_table::Table<fs::File<tc_btree::Node>, fs::Dir, Txn>;
 pub type TableIndex = tc_table::TableIndex<fs::File<tc_btree::Node>, fs::Dir, Txn>;
 
-const PREFIX: PathLabel = path_label(&["state", "collection"]);
+#[cfg(feature = "tensor")]
+pub type Tensor = tc_tensor::Tensor<fs::File<afarray::Array>, fs::Dir, Txn>;
+#[cfg(feature = "tensor")]
+pub type DenseTensor<B> = tc_tensor::DenseTensor<fs::File<afarray::Array>, fs::Dir, Txn, B>;
+#[cfg(feature = "tensor")]
+pub type DenseTensorFile = tc_tensor::BlockListFile<fs::File<afarray::Array>, fs::Dir, Txn>;
+
+pub const PREFIX: PathLabel = path_label(&["state", "collection"]);
 
 /// The [`Class`] of a [`Collection`].
 #[derive(Clone, Copy, Eq, PartialEq)]
 pub enum CollectionType {
     BTree(BTreeType),
     Table(TableType),
+    #[cfg(feature = "tensor")]
+    Tensor(TensorType),
 }
 
 impl Class for CollectionType {}
@@ -47,6 +62,8 @@ impl NativeClass for CollectionType {
             match path[2].as_str() {
                 "btree" => BTreeType::from_path(path).map(Self::BTree),
                 "table" => TableType::from_path(path).map(Self::Table),
+                #[cfg(feature = "tensor")]
+                "tensor" => TensorType::from_path(path).map(Self::Tensor),
                 _ => None,
             }
         } else {
@@ -56,8 +73,10 @@ impl NativeClass for CollectionType {
 
     fn path(&self) -> TCPathBuf {
         match self {
-            Self::BTree(btree) => btree.path(),
-            Self::Table(table) => table.path(),
+            Self::BTree(btt) => btt.path(),
+            Self::Table(tt) => tt.path(),
+            #[cfg(feature = "tensor")]
+            Self::Tensor(tt) => tt.path(),
         }
     }
 }
@@ -74,11 +93,20 @@ impl From<TableType> for CollectionType {
     }
 }
 
+#[cfg(feature = "tensor")]
+impl From<TensorType> for CollectionType {
+    fn from(tt: TensorType) -> Self {
+        Self::Tensor(tt)
+    }
+}
+
 impl fmt::Display for CollectionType {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
             Self::BTree(btt) => fmt::Display::fmt(btt, f),
             Self::Table(tt) => fmt::Display::fmt(tt, f),
+            #[cfg(feature = "tensor")]
+            Self::Tensor(tt) => fmt::Display::fmt(tt, f),
         }
     }
 }
@@ -88,6 +116,8 @@ impl fmt::Display for CollectionType {
 pub enum Collection {
     BTree(BTree),
     Table(Table),
+    #[cfg(feature = "tensor")]
+    Tensor(Tensor),
 }
 
 impl Instance for Collection {
@@ -97,6 +127,8 @@ impl Instance for Collection {
         match self {
             Self::BTree(btree) => CollectionType::BTree(btree.class()),
             Self::Table(table) => CollectionType::Table(table.class()),
+            #[cfg(feature = "tensor")]
+            Self::Tensor(tensor) => CollectionType::Tensor(tensor.class()),
         }
     }
 }
@@ -125,6 +157,37 @@ impl From<TableIndex> for Collection {
     }
 }
 
+#[cfg(feature = "tensor")]
+impl From<Tensor> for Collection {
+    fn from(tensor: Tensor) -> Self {
+        Self::Tensor(tensor)
+    }
+}
+
+#[cfg(feature = "tensor")]
+impl<B: DenseAccess<fs::File<afarray::Array>, fs::Dir, Txn>> From<DenseTensor<B>> for Collection {
+    fn from(tensor: DenseTensor<B>) -> Self {
+        Self::Tensor(tensor.into())
+    }
+}
+
+#[cfg(feature = "tensor")]
+impl safecast::TryCastFrom<Collection> for Tensor {
+    fn can_cast_from(collection: &Collection) -> bool {
+        match collection {
+            Collection::Tensor(_) => true,
+            _ => false,
+        }
+    }
+
+    fn opt_cast_from(collection: Collection) -> Option<Self> {
+        match collection {
+            Collection::Tensor(tensor) => Some(tensor),
+            _ => None,
+        }
+    }
+}
+
 /// A [`de::Visitor`] used to deserialize a [`Collection`].
 pub struct CollectionVisitor {
     txn: Txn,
@@ -140,6 +203,8 @@ impl CollectionVisitor {
         class: CollectionType,
         access: &mut A,
     ) -> Result<Collection, A::Error> {
+        debug!("deserialize Collection");
+
         match class {
             CollectionType::BTree(_) => {
                 let file = self
@@ -154,7 +219,18 @@ impl CollectionVisitor {
                     .map_ok(Collection::BTree)
                     .await
             }
+
             CollectionType::Table(_) => access.next_value(self.txn).map_ok(Collection::Table).await,
+
+            #[cfg(feature = "tensor")]
+            CollectionType::Tensor(tt) => match tt {
+                TensorType::Dense => {
+                    debug!("deserialize Tensor");
+
+                    let tensor: DenseTensor<DenseTensorFile> = access.next_value(self.txn).await?;
+                    Ok(Collection::Tensor(tensor.into()))
+                }
+            },
         }
     }
 }
@@ -198,6 +274,8 @@ impl<'en> IntoView<'en, fs::Dir> for Collection {
         match self {
             Self::BTree(btree) => btree.into_view(txn).map_ok(CollectionView::BTree).await,
             Self::Table(table) => table.into_view(txn).map_ok(CollectionView::Table).await,
+            #[cfg(feature = "tensor")]
+            Self::Tensor(tensor) => tensor.into_view(txn).map_ok(CollectionView::Tensor).await,
         }
     }
 }
@@ -207,6 +285,8 @@ impl fmt::Display for Collection {
         match self {
             Self::BTree(btree) => fmt::Display::fmt(btree, f),
             Self::Table(table) => fmt::Display::fmt(table, f),
+            #[cfg(feature = "tensor")]
+            Self::Tensor(tensor) => fmt::Display::fmt(tensor, f),
         }
     }
 }
@@ -215,6 +295,8 @@ impl fmt::Display for Collection {
 pub enum CollectionView<'en> {
     BTree(BTreeView<'en>),
     Table(TableView<'en>),
+    #[cfg(feature = "tensor")]
+    Tensor(TensorView<'en>),
 }
 
 impl<'en> en::IntoStream<'en> for CollectionView<'en> {
@@ -225,6 +307,10 @@ impl<'en> en::IntoStream<'en> for CollectionView<'en> {
         match self {
             Self::BTree(btree) => map.encode_entry(BTreeType::default().path(), btree),
             Self::Table(table) => map.encode_entry(TableType::default().path(), table),
+            #[cfg(feature = "tensor")]
+            Self::Tensor(tensor) => match tensor {
+                TensorView::Dense(tensor) => map.encode_entry(TensorType::Dense.path(), tensor),
+            },
         }?;
         map.end()
     }
