@@ -6,6 +6,8 @@ use std::fmt;
 use std::path::PathBuf;
 use std::pin::Pin;
 
+#[cfg(feature = "tensor")]
+use afarray::Array;
 use async_trait::async_trait;
 use futures::future::{join_all, Future, TryFutureExt};
 use log::debug;
@@ -13,6 +15,8 @@ use uuid::Uuid;
 
 use tc_btree::Node;
 use tc_error::*;
+#[cfg(feature = "tensor")]
+use tc_tensor::TensorType;
 use tc_transact::fs::{self, BlockData};
 use tc_transact::lock::{Mutable, TxnLock};
 use tc_transact::{Transact, TxnId};
@@ -32,6 +36,9 @@ pub enum FileEntry {
     BTree(File<Node>),
     Chain(File<ChainBlock>),
     Value(File<Value>),
+
+    #[cfg(feature = "tensor")]
+    Tensor(File<Array>),
 }
 
 impl FileEntry {
@@ -47,6 +54,11 @@ impl FileEntry {
             StateType::Collection(ct) => match ct {
                 CollectionType::BTree(_) => Ok(Self::BTree(File::new(cache, path, Node::ext()))),
                 CollectionType::Table(tt) => Err(err(tt)),
+
+                #[cfg(feature = "tensor")]
+                CollectionType::Tensor(tt) => match tt {
+                    TensorType::Dense => Ok(Self::Tensor(File::new(cache, path, Array::ext()))),
+                },
             },
             StateType::Chain(_) => Ok(Self::Chain(File::new(cache, path, ChainBlock::ext()))),
             StateType::Scalar(st) => match st {
@@ -70,6 +82,11 @@ impl FileEntry {
                 let file = File::load(cache.clone(), path, contents)?;
                 Ok(FileEntry::Chain(file))
             }
+            #[cfg(feature = "tensor")]
+            tc_tensor::EXT => {
+                let file = File::load(cache.clone(), path, contents)?;
+                Ok(FileEntry::Tensor(file))
+            }
             VALUE_EXT => {
                 let file = File::load(cache.clone(), path, contents)?;
                 Ok(FileEntry::Value(file))
@@ -88,6 +105,21 @@ impl From<File<ChainBlock>> for FileEntry {
     }
 }
 
+#[cfg(feature = "tensor")]
+impl TryFrom<FileEntry> for File<Array> {
+    type Error = TCError;
+
+    fn try_from(entry: FileEntry) -> TCResult<Self> {
+        match entry {
+            FileEntry::Tensor(file) => Ok(file),
+            other => Err(TCError::bad_request(
+                "expected a Tensor file but found",
+                other,
+            )),
+        }
+    }
+}
+
 impl TryFrom<FileEntry> for File<ChainBlock> {
     type Error = TCError;
 
@@ -142,6 +174,9 @@ impl fmt::Display for FileEntry {
             Self::BTree(btree) => fmt::Display::fmt(btree, f),
             Self::Chain(chain) => fmt::Display::fmt(chain, f),
             Self::Value(value) => fmt::Display::fmt(value, f),
+
+            #[cfg(feature = "tensor")]
+            Self::Tensor(tensor) => fmt::Display::fmt(tensor, f),
         }
     }
 }
@@ -380,6 +415,9 @@ impl Transact for Dir {
                     FileEntry::BTree(file) => file.commit(txn_id),
                     FileEntry::Chain(file) => file.commit(txn_id),
                     FileEntry::Value(file) => file.commit(txn_id),
+
+                    #[cfg(feature = "tensor")]
+                    FileEntry::Tensor(file) => file.commit(txn_id),
                 },
             }))
             .await;
@@ -397,6 +435,9 @@ impl Transact for Dir {
                     FileEntry::BTree(file) => file.finalize(txn_id),
                     FileEntry::Chain(file) => file.finalize(txn_id),
                     FileEntry::Value(file) => file.finalize(txn_id),
+
+                    #[cfg(feature = "tensor")]
+                    FileEntry::Tensor(file) => file.finalize(txn_id),
                 },
             }))
             .await;
