@@ -5,19 +5,17 @@ use std::fmt;
 use std::pin::Pin;
 
 use bytes::Bytes;
-use futures::future::{Future, TryFutureExt};
+use futures::future::Future;
 use log::debug;
-use safecast::{TryCastFrom, TryCastInto};
+use safecast::*;
 
 use tc_error::*;
-use tc_transact::fs::Dir;
-use tc_transact::Transaction;
 use tcgeneric::*;
 
 use crate::cluster::Cluster;
-use crate::collection::{BTreeFile, Collection, CollectionType, TableIndex};
+use crate::collection::CollectionType;
 use crate::object::InstanceExt;
-use crate::route::Public;
+use crate::route::{Public, Route};
 use crate::scalar::*;
 use crate::state::*;
 use crate::txn::*;
@@ -73,6 +71,21 @@ impl Kernel {
             );
 
             cluster.get(&txn, suffix, key).await
+        } else if path[..2] == crate::collection::PREFIX[..] {
+            let mut i = path.len() - 1;
+            while i > 0 {
+                if let Some(class) = CollectionType::from_path(&path[..i]) {
+                    if let Some(handler) = class.route(&path[i..]) {
+                        if let Some(handler) = handler.get() {
+                            return handler(txn.clone(), key).await;
+                        }
+                    }
+                }
+
+                i -= 1;
+            }
+
+            Err(TCError::not_found(TCPath::from(path)))
         } else if &path[0] == "error" && path.len() == 2 {
             let message = String::try_cast_from(key, |v| {
                 TCError::bad_request("cannot cast into error message string from", v)
@@ -274,30 +287,18 @@ fn execute<
 
 async fn construct_state(txn: &Txn, class: StateType, value: Value) -> TCResult<State> {
     match class {
-        StateType::Collection(class) => match class {
-            CollectionType::BTree(btt) => {
-                let schema = tc_btree::RowSchema::try_cast_from(value, |v| {
-                    TCError::bad_request("invalid BTree schema", v)
-                })?;
-
-                let file = txn.context().create_file_tmp(*txn.id(), btt).await?;
-                BTreeFile::create(file, schema, *txn.id())
-                    .map_ok(Collection::from)
-                    .map_ok(State::from)
-                    .await
+        StateType::Collection(class) => {
+            if let Some(handler) = class.route(&[]) {
+                if let Some(handler) = handler.get() {
+                    return handler(txn.clone(), value).await;
+                }
             }
-            CollectionType::Table(_) => {
-                let schema = tc_table::TableSchema::try_cast_from(value, |v| {
-                    TCError::bad_request("invalid Table schema", v)
-                })?;
 
-                let dir = txn.context().create_dir_tmp(*txn.id()).await?;
-                TableIndex::create(schema, &dir, *txn.id())
-                    .map_ok(Collection::from)
-                    .map_ok(State::from)
-                    .await
-            }
-        },
+            Err(TCError::not_implemented(format!(
+                "constructor handler for {}",
+                class
+            )))
+        }
         StateType::Chain(ct) => Err(TCError::not_implemented(format!("GET {}", ct))),
         StateType::Map => {
             let value = Tuple::<(Id, Value)>::try_cast_from(value, |v| {
