@@ -38,6 +38,8 @@ impl<'a> Handler<'a> for SubjectHandler<'a> {
                 match self.subject {
                     Subject::BTree(btree) => btree.get(&txn, self.path, key).await,
                     Subject::Table(table) => Public::get(table, &txn, self.path, key).await,
+                    #[cfg(feature = "tensor")]
+                    Subject::Tensor(tensor) => tensor.get(&txn, self.path, key).await,
                     Subject::Value(file) => {
                         let value = file.read_block(*txn.id(), SUBJECT.into()).await?;
                         if self.path.is_empty() {
@@ -57,6 +59,8 @@ impl<'a> Handler<'a> for SubjectHandler<'a> {
                 match self.subject {
                     Subject::BTree(btree) => btree.put(&txn, self.path, key, value).await,
                     Subject::Table(table) => table.put(&txn, self.path, key, value).await,
+                    #[cfg(feature = "tensor")]
+                    Subject::Tensor(tensor) => tensor.put(&txn, self.path, key, value).await,
                     Subject::Value(file) if self.path.is_empty() => {
                         let mut subject = file.write_block(*txn.id(), SUBJECT.into()).await?;
 
@@ -87,6 +91,8 @@ impl<'a> Handler<'a> for SubjectHandler<'a> {
                 match self.subject {
                     Subject::BTree(btree) => btree.post(&txn, self.path, params).await,
                     Subject::Table(table) => table.post(&txn, self.path, params).await,
+                    #[cfg(feature = "tensor")]
+                    Subject::Tensor(tensor) => tensor.post(&txn, self.path, params).await,
                     Subject::Value(file) => {
                         let subject = file.read_block(*txn.id(), SUBJECT.into()).await?;
                         subject.post(&txn, self.path, params).await
@@ -102,6 +108,8 @@ impl<'a> Handler<'a> for SubjectHandler<'a> {
                 match self.subject {
                     Subject::BTree(btree) => btree.delete(&txn, self.path, key).await,
                     Subject::Table(table) => Public::delete(table, &txn, self.path, key).await,
+                    #[cfg(feature = "tensor")]
+                    Subject::Tensor(tensor) => tensor.delete(&txn, self.path, key).await,
                     Subject::Value(file) if self.path.is_empty() => {
                         let mut subject = file.write_block(*txn.id(), SUBJECT.into()).await?;
                         *subject = Value::None;
@@ -157,7 +165,7 @@ impl<'a> Handler<'a> for AppendHandler<'a> {
 
                         let path = self.path.to_vec().into();
                         self.chain
-                            .append_put(*txn.id(), path, key.clone(), value.clone())
+                            .append_put(txn.clone(), path, key.clone(), value.clone())
                             .await?;
 
                         put_handler(txn, key, value).await
