@@ -352,6 +352,12 @@ impl From<Value> for Scalar {
     }
 }
 
+impl From<bool> for Scalar {
+    fn from(b: bool) -> Self {
+        Self::Value(b.into())
+    }
+}
+
 impl From<u64> for Scalar {
     fn from(n: u64) -> Self {
         Self::Value(n.into())
@@ -649,6 +655,22 @@ impl TryCastFrom<Scalar> for bool {
     }
 }
 
+impl TryCastFrom<Scalar> for u64 {
+    fn can_cast_from(scalar: &Scalar) -> bool {
+        match scalar {
+            Scalar::Value(value) => Self::can_cast_from(value),
+            _ => false,
+        }
+    }
+
+    fn opt_cast_from(scalar: Scalar) -> Option<Self> {
+        match scalar {
+            Scalar::Value(value) => Self::opt_cast_from(value),
+            _ => None,
+        }
+    }
+}
+
 impl TryCastFrom<Scalar> for Value {
     fn can_cast_from(scalar: &Scalar) -> bool {
         match scalar {
