@@ -106,6 +106,13 @@ impl From<TableType> for StateType {
     }
 }
 
+#[cfg(feature = "tensor")]
+impl From<TensorType> for StateType {
+    fn from(tt: TensorType) -> Self {
+        Self::Collection(tt.into())
+    }
+}
+
 impl From<ValueType> for StateType {
     fn from(vt: ValueType) -> Self {
         Self::Scalar(vt.into())
@@ -311,6 +318,13 @@ impl From<Table> for State {
     }
 }
 
+#[cfg(feature = "tensor")]
+impl From<Tensor> for State {
+    fn from(tensor: Tensor) -> Self {
+        Self::Collection(tensor.into())
+    }
+}
+
 impl From<Tuple<State>> for State {
     fn from(tuple: Tuple<State>) -> Self {
         Self::Tuple(tuple)
@@ -347,6 +361,12 @@ impl From<Value> for State {
     }
 }
 
+impl From<bool> for State {
+    fn from(b: bool) -> Self {
+        Self::Scalar(b.into())
+    }
+}
+
 impl From<u64> for State {
     fn from(n: u64) -> Self {
         Self::Scalar(n.into())
@@ -614,6 +634,23 @@ impl TryCastFrom<State> for Scalar {
     }
 }
 
+#[cfg(feature = "tensor")]
+impl TryCastFrom<State> for Tensor {
+    fn can_cast_from(state: &State) -> bool {
+        match state {
+            State::Collection(collection) => Self::can_cast_from(collection),
+            _ => false,
+        }
+    }
+
+    fn opt_cast_from(state: State) -> Option<Self> {
+        match state {
+            State::Collection(collection) => Self::opt_cast_from(collection),
+            _ => None,
+        }
+    }
+}
+
 impl TryCastFrom<State> for TCPathBuf {
     fn can_cast_from(state: &State) -> bool {
         match state {
@@ -652,6 +689,22 @@ impl TryCastFrom<State> for Value {
     }
 }
 
+impl TryCastFrom<State> for Number {
+    fn can_cast_from(state: &State) -> bool {
+        match state {
+            State::Scalar(scalar) => Self::can_cast_from(scalar),
+            _ => false,
+        }
+    }
+
+    fn opt_cast_from(state: State) -> Option<Self> {
+        match state {
+            State::Scalar(scalar) => Self::opt_cast_from(scalar),
+            _ => None,
+        }
+    }
+}
+
 impl fmt::Display for State {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
@@ -788,6 +841,7 @@ impl<'a> de::Visitor for StateVisitor {
             if key.starts_with('/') {
                 if let Ok(path) = TCPathBuf::from_str(&key) {
                     debug!("is {} a classpath?", path);
+
                     if let Some(class) = StateType::from_path(&path) {
                         debug!("deserialize instance of {}...", class);
                         return self.visit_map_value(class, &mut access).await;
